# Generated by Django 2.1.14 on 2020-09-23 10:49

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0124_auto_20200917_1922'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='ticket',
            name='predecessor_id',
        ),
        migrations.AddField(
            model_name='ticket',
            name='phase_predecessor',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djconnectwise.ProjectPhase'),
        ),
        migrations.AddField(
            model_name='ticket',
            name='ticket_predecessor',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djconnectwise.Ticket'),
        ),
    ]



# Generated by Django 4.2.11 on 2024-05-23 10:10

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0179_projectphase_required_date'),
        ('djconnectwise', '0179_ticket_merged_parent_alter_ticket_ticket_predecessor'),
    ]

    operations = [
    ]



# Generated by Django 4.2.16 on 2024-10-30 15:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0186_configurationstatus_configurationtype_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='member',
            name='title',
            field=models.CharField(blank=True, max_length=250, null=True),
        ),
    ]



# Generated by Django 4.2.16 on 2025-04-02 16:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0189_remove_sla_calendar_remove_slapriority_priority_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='project',
            name='percent_complete',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0050_servicenote'),
    ]

    operations = [
        migrations.CreateModel(
            name='OpportunityNote',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, verbose_name='ID', primary_key=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(verbose_name='created', auto_now_add=True)),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(verbose_name='modified', auto_now=True)),
                ('text', models.TextField(blank=True, null=True, max_length=2000)),
                ('opportunity', models.ForeignKey(to='djconnectwise.Opportunity', on_delete=models.CASCADE)),
            ],
            options={
                'verbose_name_plural': 'Opportunity Notes',
                'ordering': ('id',),
            },
        ),
    ]



# Generated by Django 4.2.11 on 2024-05-23 12:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0180_alter_companysite_inactive'),
        ('djconnectwise', '0180_merge_20240523_1010'),
    ]

    operations = [
    ]



# Generated by Django 4.0.10 on 2023-06-16 19:47

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0173_rename_defaultflag_companynotetype_default_flag'),
    ]

    operations = [
        migrations.CreateModel(
            name='CompanyTeamRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('accountManagerFlag', models.BooleanField()),
                ('techFlag', models.BooleanField()),
                ('salesFlag', models.BooleanField()),
            ],
            options={
                'ordering': ('name',),
            },
        ),
        migrations.CreateModel(
            name='CompanyTeam',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('accountManagerFlag', models.BooleanField()),
                ('techFlag', models.BooleanField()),
                ('salesFlag', models.BooleanField()),
                ('company', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djconnectwise.company')),
                ('contact', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djconnectwise.contact')),
                ('location', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djconnectwise.location')),
                ('member', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djconnectwise.member')),
                ('team_role', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djconnectwise.companyteamrole')),
            ],
            options={
                'ordering': ('id',),
            },
        ),
        migrations.CreateModel(
            name='CompanyTeamRoleTracker',
            fields=[
            ],
            options={
                'db_table': 'djconnectwise_companyteamrole',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djconnectwise.companyteamrole',),
        ),
        migrations.CreateModel(
            name='CompanyTeamTracker',
            fields=[
            ],
            options={
                'db_table': 'djconnectwise_companyteam',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djconnectwise.companyteam',),
        ),
    ]



# Generated by Django 4.0 on 2022-02-24 16:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0159_alter_ticket_budget_hours'),
    ]

    operations = [
        migrations.AlterField(
            model_name='project',
            name='budget_hours',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=12, null=True),
        ),
    ]



# Generated by Django 4.2.11 on 2024-06-05 16:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0183_companytype_default_flag_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContactType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(max_length=250)),
                ('default_flag', models.BooleanField()),
                ('service_alert_flag', models.BooleanField(blank=True, null=True)),
                ('service_alert_message', models.TextField(blank=True, null=True)),
            ],
            options={
                'ordering': ('description',),
            },
        ),
        migrations.CreateModel(
            name='ContactTypeTracker',
            fields=[
            ],
            options={
                'db_table': 'djconnectwise_contacttype',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djconnectwise.contacttype',),
        ),
    ]



# Generated by Django 4.0.7 on 2023-03-31 10:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0169_connectwiseboard_time_entry_discussion_flag_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='ticket',
            name='predecessor_closed_flag',
            field=models.BooleanField(default=False),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0005_auto_20170225_0101'),
    ]

    operations = [
        migrations.AlterField(
            model_name='callbackentry',
            name='description',
            field=models.CharField(max_length=100, null=True, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
VERSION = (1, 9, 0, 'final')

# pragma: no cover
if VERSION[-1] != "final":
    __version__ = '.'.join(map(str, VERSION))
else:
    # pragma: no cover
    __version__ = '.'.join(map(str, VERSION[:-1]))

default_app_config = 'djconnectwise.apps.DjangoConnectwiseConfig'



# Generated by Django 4.2.11 on 2024-05-17 15:19

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0178_alter_ticket_work_role_alter_ticket_work_type'),
    ]

    operations = [
        migrations.CreateModel(
            name='CompanySite',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('inactive', models.BooleanField(default=False)),
                ('company', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djconnectwise.company')),
            ],
            options={
                'ordering': ('id',),
            },
        ),
        migrations.CreateModel(
            name='CompanySiteTracker',
            fields=[
            ],
            options={
                'db_table': 'djconnectwise_companysite',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djconnectwise.companysite',),
        ),
        migrations.AddField(
            model_name='ticket',
            name='company_site',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djconnectwise.companysite'),
        ),
    ]



# Generated by Django 4.2.11 on 2024-05-14 20:34

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0178_alter_ticket_work_role_alter_ticket_work_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='ticket',
            name='merged_parent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='merged_parent_ticket', to='djconnectwise.ticket'),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='ticket_predecessor',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='ticket_predecessor_ticket', to='djconnectwise.ticket'),
        ),
    ]



# Generated by Django 3.1.7 on 2021-11-05 09:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0157_auto_20211102_1328'),
    ]

    operations = [
        migrations.AlterField(
            model_name='timeentry',
            name='notes',
            field=models.TextField(blank=True, max_length=50000, null=True),
        ),
    ]



# Generated by Django 4.0.4 on 2022-08-05 17:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0165_sourcetracker'),
    ]

    operations = [
        migrations.RenameField(
            model_name='ticket',
            old_name='initial',
            new_name='initial_description',
        ),
        migrations.AddField(
            model_name='ticket',
            name='is_issue_flag',
            field=models.BooleanField(null=True),
        ),
    ]



# Generated by Django 4.0.4 on 2022-06-22 07:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0162_source'),
    ]

    operations = [
        migrations.AddField(
            model_name='ticket',
            name='initial',
            field=models.CharField(blank=True, max_length=5000, null=True),
        ),
    ]



# Generated by Django 4.0.10 on 2023-10-18 11:43

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0176_project_billing_method'),
    ]

    operations = [
        migrations.AddField(
            model_name='ticket',
            name='work_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='djconnectwise.workrole'),
        ),
        migrations.AddField(
            model_name='ticket',
            name='work_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='djconnectwise.worktype'),
        ),
    ]



# Generated by Django 2.0 on 2018-09-25 16:18

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0075_auto_20180910_1629'),
    ]

    operations = [
        migrations.CreateModel(
            name='Holiday',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('all_day_flag', models.BooleanField(default=False)),
                ('date', models.DateField(blank=True, null=True)),
                ('start_time', models.TimeField(blank=True, null=True)),
                ('end_time', models.TimeField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='HolidayList',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
            ],
        ),
        migrations.AddField(
            model_name='holiday',
            name='holiday_list',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djconnectwise.HolidayList'),
        ),
        migrations.AddField(
            model_name='calendar',
            name='holiday_list',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djconnectwise.HolidayList'),
        ),
    ]



# Generated by Django 3.1.7 on 2021-04-27 22:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0148_auto_20210317_1424'),
    ]

    operations = [
        migrations.AlterField(
            model_name='servicenote',
            name='text',
            field=models.TextField(blank=True, max_length=4000, null=True),
        ),
    ]



# Generated by Django 4.2.16 on 2025-02-21 10:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0188_projectrole_projectroletracker_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='sla',
            name='calendar',
        ),
        migrations.RemoveField(
            model_name='slapriority',
            name='priority',
        ),
        migrations.RemoveField(
            model_name='slapriority',
            name='sla',
        ),
        migrations.DeleteModel(
            name='SlaPriorityTracker',
        ),
        migrations.DeleteModel(
            name='SlaTracker',
        ),
        migrations.RemoveField(
            model_name='ticket',
            name='date_resolved_utc',
        ),
        migrations.RemoveField(
            model_name='ticket',
            name='date_resplan_utc',
        ),
        migrations.RemoveField(
            model_name='ticket',
            name='date_responded_utc',
        ),
        migrations.RemoveField(
            model_name='ticket',
            name='do_not_escalate_date',
        ),
        migrations.RemoveField(
            model_name='ticket',
            name='minutes_waiting',
        ),
        migrations.RemoveField(
            model_name='ticket',
            name='res_plan_mins',
        ),
        migrations.RemoveField(
            model_name='ticket',
            name='resolve_mins',
        ),
        migrations.RemoveField(
            model_name='ticket',
            name='respond_mins',
        ),
        migrations.RemoveField(
            model_name='ticket',
            name='sla',
        ),
        migrations.DeleteModel(
            name='Sla',
        ),
        migrations.DeleteModel(
            name='SlaPriority',
        ),
    ]



# Generated by Django 2.1 on 2019-07-22 14:44

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0104_auto_20190722_1232'),
    ]

    operations = [
        migrations.AddField(
            model_name='activity',
            name='status',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djconnectwise.ActivityStatus'),
        ),
        migrations.AddField(
            model_name='activity',
            name='type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djconnectwise.ActivityType'),
        ),
    ]



# Generated by Django 4.2.16 on 2025-01-31 15:14

from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0187_member_title'),
    ]

    operations = [
        migrations.CreateModel(
            name='ProjectRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('name', models.CharField(max_length=30)),
                ('manager_role_flag', models.BooleanField(default=False)),
                ('default_contact_flag', models.BooleanField(default=False)),
            ],
            options={
                'verbose_name_plural': 'Project role',
                'ordering': ('name',),
            },
        ),
        migrations.CreateModel(
            name='ProjectRoleTracker',
            fields=[
            ],
            options={
                'db_table': 'djconnectwise_projectrole',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djconnectwise.projectrole',),
        ),
        migrations.AddField(
            model_name='projectteammember',
            name='project_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='djconnectwise.projectrole'),
        ),
    ]



# Generated by Django 4.2.11 on 2024-05-22 15:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0179_companysite_companysitetracker_ticket_company_site'),
    ]

    operations = [
        migrations.AlterField(
            model_name='companysite',
            name='inactive',
            field=models.BooleanField(blank=True, default=False, null=True),
        ),
    ]



# Generated by Django 2.0 on 2019-01-25 11:09

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0081_auto_20181220_0917'),
    ]

    operations = [
        migrations.AlterField(
            model_name='opportunity',
            name='priority',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='djconnectwise.OpportunityPriority'),
        ),
    ]



# Generated by Django 2.1 on 2019-07-16 14:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0091_workrole'),
        ('djconnectwise', '0092_remove_ticket_work_type'),
    ]

    operations = [
    ]



# Generated by Django 3.1.4 on 2021-01-12 14:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0143_contactcommunication_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='member',
            name='license_class',
            field=models.CharField(blank=True, choices=[('F', 'Full license'), ('A', 'API license'), ('C', 'StreamlineIT license'), ('X', 'Subcontractor license')], db_index=True, max_length=20, null=True),
        ),
    ]



import re
import hashlib
from io import BytesIO
from datetime import datetime, timedelta, timezone

from PIL import Image, ImageOps
from django.conf import settings
from django.core.files.base import ContentFile
from django.core.files.storage import default_storage

_underscorer1 = re.compile(r'(.)([A-Z][a-z]+)')
_underscorer2 = re.compile(r'([a-z0-9])([A-Z])')
FILENAME_EXTENSION_RE = re.compile(r'\.([\w]*)$')

# Any more than this, the days of the week will likely have all repeated.
YEARS_TO_CHECK = 6

SLA_MAX_DAYS = 40


def camel_to_snake(s):
    """
    Is it ironic that this function is written in camel case, yet it
    converts to snake case? hmm..
    """
    subbed = _underscorer1.sub(r'\1_\2', s)
    return _underscorer2.sub(r'\1_\2', subbed).lower()


def snake_to_camel(snake_case_text):
    tokens = snake_case_text.split('_')
    return ''.join(word.capitalize() for word in tokens)


def get_hash(content):
    """Return the hex SHA-1 hash of the given content."""
    return hashlib.sha1(content).hexdigest()


def get_filename_extension(filename):
    """From the given filename, return the extension,
    or None if it can't be parsed.
    """
    m = FILENAME_EXTENSION_RE.search(filename)
    return m.group(1) if m else None


def generate_filename(size, current_filename, extension):
    img_dimensions = 'x'.join([str(i) for i in size])
    filename = '{}{}.{}'.format(current_filename, img_dimensions, extension)
    return filename


def remove_thumbnail(avatar_filename):
    thumbnail_size = {
        'avatar': (80, 80),
        'micro_avatar': (20, 20),
    }
    # This deletes the image name from DB field
    # and also removes thumbnails from storage.
    extension = get_filename_extension(avatar_filename)
    for size in thumbnail_size:
        filename = generate_filename(thumbnail_size[size],
                                     avatar_filename, extension)
        default_storage.delete(filename)

    default_storage.delete(avatar_filename)


def generate_thumbnail(avatar, size, extension, filename):
    filename = generate_filename(size, filename, extension)
    avatar_image = Image.open(BytesIO(avatar))
    thumbnail = ImageOps.fit(avatar_image, size, Image.LANCZOS)

    # For now just change the extension to jpeg
    # https://stackoverflow.com/questions/37048807/python-image-library-and-keyerror-jpg
    if extension.lower() == 'jpg':
        extension = 'jpeg'

    byte_stream = BytesIO()
    thumbnail.save(byte_stream, format=extension)
    avatar_file = ContentFile(byte_stream.getvalue())

    return avatar_file, filename


def generate_image_url(company_id, guid):
    if not guid:
        return None

    return settings.CONNECTWISE_SERVER_URL + \
        '/v4_6_release/api/inlineimages/' + company_id + '/' + guid


def parse_sla_status(sla_status, date_created):
    """
    Parse the SLA status string from ConnectWise into a datetime object and
    SLA stage name.

    :param sla_status: The SLA status string from ConnectWise.
    :param date_created: The creation date of the ticket, UTC

    :return: A tuple containing the SLA stage name and the datetime object,
             or the SLA stage and None if the SLA status is Waiting or
             Resolved.
    """

    def _parse_check_date(check_date_string):
        for fmt in ['%Y %m %d %I:%M %p', '%Y %d %m %I:%M %p']:
            try:
                return datetime.strptime(check_date_string, fmt)
            except ValueError as e:
                if "day is out of range for month" in str(e):
                    # Likely Leap day on a non-leap year, return None
                    return None

    def _check_ambiguity(ambiguous_date, dow):
        if ambiguous_date.month > 12 or ambiguous_date.day > 12:
            # Can't be ambiguous, return
            return ambiguous_date

        # Dates can be ambiguous like 3/5 or 5/3, but SLAs can't be further
        # than ~40 days out, so reverse the date if the check date is over
        # 40 days from today's date
        today_utc = datetime.now(timezone.utc)
        days_delta = (ambiguous_date - today_utc).days

        # If the check_date is more than 40 days from today, reverse the date
        if days_delta > SLA_MAX_DAYS:
            ambiguous_date = check_date.replace(
                day=ambiguous_date.month, month=ambiguous_date.day)

        elif ambiguous_date.strftime("%a") != dow:
            # Check if the date's day-of-week matches the expected day.
            # If not, reverse the date. This handles cases if a ticket
            # DOES have an expired sla, and it expired several months
            # earlier.

            ambiguous_date = check_date.replace(
                day=ambiguous_date.month, month=ambiguous_date.day)

        return ambiguous_date

    # Regex matches day-of-week, month/day, time, and timezone offset.
    # Example SLA data from CW: "Respond by Mon 01/27 4:00 PM UTC-08"
    pattern = (r'\w+\sby\s(\w{3})\s(\d{2})/(\d{2})\s(\d{1,2}:\d{2})\s'
               r'(AM|PM)\sUTC([+-]\d{1,2})')
    match = re.search(pattern, sla_status)

    if not match:
        # There is no date, so it is Waiting or Resolved. Just return the
        # status.
        return sla_status.strip(), None

    day_of_week, month, day, time_str, am_pm, tz_offset_str = match.groups()
    month = int(month)
    day = int(day)
    tz_offset_hours = int(tz_offset_str)
    tz_info = timezone(timedelta(hours=tz_offset_hours))

    utc_date = None

    # We'll iterate over several years, starting with date_created.year (as
    # the SLA can't be before the ticket was created).
    for check_year in range(
            date_created.year, date_created.year + YEARS_TO_CHECK):

        # Create with string because this makes handling AM/PM much easier.
        # Otherwise, we have to manually convert 12-hour to 24-hour, then
        # handle the case when it crosses midnight, it's a whole thing.

        date_string = f"{check_year} {month:02d} {day:02d} {time_str} {am_pm}"

        # Nested function to make this easier to read.
        check_date = _parse_check_date(date_string)

        check_date = check_date.replace(tzinfo=tz_info)

        # Dates can be ambiguous like 3/5 or 5/3, handle it
        check_date = _check_ambiguity(check_date, day_of_week)

        # Skip dates that occur before the ticket was created, or leap
        # days that don't exist in the year.
        if not check_date or check_date < date_created:
            continue

        # Check if the date's day-of-week matches the expected day.
        if check_date.strftime("%a") == day_of_week:
            # Convert to UTC
            utc_date = check_date.astimezone(timezone.utc)
            break

    # If we reach this point, we couldn't find a valid date. Just return the
    # stage with utc date as None I guess. There isn't really much we can do,
    # and if you have a 6 year old SLA, that's a you problem.
    stage = sla_status.split()[0].lower()
    return stage, utc_date


class DjconnectwiseSettings:

    def get_settings(self):
        # Make some defaults
        request_settings = {
            'timeout': 30.0,
            'batch_size': 50,
            'max_attempts': 3,
            'max_url_length': 2000,
            'schedule_entry_conditions_size': 0,
            'response_version': '2020.4',
            'sync_child_tickets': True,
            'sync_time_and_note_entries': True,
            'sync_contact_communications': True,
            'keep_closed_ticket_days': 0,
            'keep_closed_status_board_ids': 0,
            'company_exclude_status_ids': '',
            'send_naive_datetimes': True,
        }

        if hasattr(settings, 'DJCONNECTWISE_CONF_CALLABLE'):
            request_settings.update(settings.DJCONNECTWISE_CONF_CALLABLE())

        return request_settings



# Generated by Django 4.2.11 on 2024-06-05 20:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0184_contacttype_contacttypetracker'),
    ]

    operations = [
        migrations.AddField(
            model_name='contact',
            name='type',
            field=models.ManyToManyField(to='djconnectwise.contacttype'),
        ),
    ]



# Generated by Django 4.2.11 on 2024-06-05 16:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0182_alter_companysite_inactive'),
    ]

    operations = [
        migrations.AddField(
            model_name='companytype',
            name='default_flag',
            field=models.BooleanField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='companytype',
            name='service_alert_flag',
            field=models.BooleanField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='companytype',
            name='service_alert_message',
            field=models.TextField(blank=True, null=True),
        ),
    ]



# Generated by Django 4.0 on 2022-05-24 13:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0160_alter_project_budget_hours'),
    ]

    operations = [
        migrations.AddField(
            model_name='activity',
            name='members',
            field=models.ManyToManyField(related_name='member_activities', through='djconnectwise.ScheduleEntry', to='djconnectwise.Member'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0032_auto_20170616_1144'),
    ]

    operations = [
        migrations.AlterField(
            model_name='syncjob',
            name='start_time',
            field=models.DateTimeField(),
        ),
    ]



# Generated by Django 2.0 on 2018-06-06 11:17

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0061_auto_20180605_1208'),
    ]

    operations = [
        migrations.CreateModel(
            name='Territory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=250, null=True)),
            ],
            options={
                'verbose_name_plural': 'Territories',
                'ordering': ('name',),
            },
        ),
        migrations.AlterModelOptions(
            name='opportunitynote',
            options={'ordering': ('-date_created', 'id'), 'verbose_name_plural': 'Opportunity notes'},
        ),
        migrations.AlterModelOptions(
            name='timeentry',
            options={'ordering': ('-time_start', 'id'), 'verbose_name_plural': 'Time entries'},
        ),
        migrations.AlterField(
            model_name='company',
            name='territory',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djconnectwise.Territory'),
        ),
    ]



# Generated by Django 4.2.11 on 2024-09-09 16:15

from django.db import migrations, models
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0185_contact_type'),
    ]

    operations = [
        migrations.CreateModel(
            name='ConfigurationStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('description', models.CharField(max_length=255)),
                ('closed_flag', models.BooleanField(default=False)),
                ('default_flag', models.BooleanField(default=False)),
            ],
            options={
                'verbose_name_plural': 'Configuration Statuses',
                'ordering': ['description'],
            },
        ),
        migrations.CreateModel(
            name='ConfigurationType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('name', models.TextField(max_length=250)),
                ('inactive_flag', models.BooleanField(default=False)),
                ('system_flag', models.BooleanField(default=False)),
            ],
            options={
                'verbose_name_plural': 'Configuration Types',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='ConfigurationStatusTracker',
            fields=[
            ],
            options={
                'db_table': 'djconnectwise_configurationstatus',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djconnectwise.configurationstatus',),
        ),
        migrations.CreateModel(
            name='ConfigurationTypeTracker',
            fields=[
            ],
            options={
                'db_table': 'djconnectwise_configurationtype',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djconnectwise.configurationtype',),
        ),
    ]



# Generated by Django 4.0 on 2022-06-16 10:22

from django.db import migrations, models
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0161_activity_members'),
    ]

    operations = [
        migrations.CreateModel(
            name='Source',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('name', models.CharField(max_length=100)),
                ('default_flag', models.BooleanField()),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
    ]



# Generated by Django 3.1.5 on 2021-02-17 14:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0145_merge_20210113_1744'),
    ]

    operations = [
        migrations.AddField(
            model_name='contact',
            name='title',
            field=models.CharField(blank=True, max_length=50, null=True),
        ),
        migrations.AddField(
            model_name='contactcommunication',
            name='extension',
            field=models.CharField(blank=True, max_length=15, null=True),
        ),
        migrations.AlterField(
            model_name='contact',
            name='first_name',
            field=models.CharField(blank=True, max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='contact',
            name='last_name',
            field=models.CharField(blank=True, max_length=30, null=True),
        ),
    ]



# Generated by Django 4.2.11 on 2024-05-21 17:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0178_alter_ticket_work_role_alter_ticket_work_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='projectphase',
            name='required_date',
            field=models.DateField(blank=True, null=True),
        ),
    ]



# Generated by Django 4.0.10 on 2023-10-25 09:00

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0177_ticket_work_role_ticket_work_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ticket',
            name='work_role',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='work_role_tickets', to='djconnectwise.workrole'),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='work_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='work_type_tickets', to='djconnectwise.worktype'),
        ),
    ]



from django.contrib import admin
from django.db import models as db_models
from django.forms import TextInput
import datetime

from . import models


@admin.register(models.SyncJob)
class SyncJobAdmin(admin.ModelAdmin):
    actions = None
    change_form_template = 'change_form.html'
    list_display = (
        'id', 'start_time', 'end_time', 'duration_or_zero', 'entity_name',
        'synchronizer_class', 'success', 'added', 'updated', 'skipped',
        'deleted', 'sync_type',
    )
    list_filter = ('sync_type', 'success', 'entity_name', 'synchronizer_class')

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def duration_or_zero(self, obj):
        """
        Return the duration, or just the string 0 (otherwise we get 0:00:00)
        """
        duration = obj.duration()
        if duration:
            # Get rid of the microseconds part
            duration_seconds = duration - datetime.timedelta(
                microseconds=duration.microseconds
            )
            return duration_seconds if duration_seconds else '0'
    duration_or_zero.short_description = 'Duration'


@admin.register(models.ConnectWiseBoard)
class ConnectWiseBoardAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'inactive')
    search_fields = ['name']
    list_filter = ('inactive',)


@admin.register(models.BoardStatus)
class BoardStatusAdmin(admin.ModelAdmin):
    list_display = ('id', 'board', 'name', 'closed_status')
    search_fields = ['name', 'board__name']


@admin.register(models.ServiceNote)
class ServiceNoteAdmin(admin.ModelAdmin):
    list_display = ('id', 'ticket', 'date_created', 'member', 'text',
                    'created_by', 'internal_flag', 'external_flag',
                    'detail_description_flag', 'internal_analysis_flag'
                    )
    search_fields = ['id', 'date_created', 'text', 'created_by']


@admin.register(models.OpportunityNote)
class OpportunityNoteAdmin(admin.ModelAdmin):
    list_display = ('id', 'opportunity', 'text')
    search_fields = ['id', 'text']


@admin.register(models.Location)
class LocationAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'where')


@admin.register(models.Member)
class MemberAdmin(admin.ModelAdmin):
    list_display = (
        'identifier', 'full_name', 'office_email', 'title', 'inactive',
        'license_class'
    )
    search_fields = (
        'identifier', 'first_name', 'last_name', 'office_email',
        'title',
    )
    list_filter = ('license_class', 'inactive', 'title',)

    def full_name(self, obj):
        return str(obj)


@admin.register(models.Territory)
class TerritoryAdmin(admin.ModelAdmin):
    list_display = ('id', 'name')
    search_fields = ['id', 'name']


@admin.register(models.Company)
class CompanyAdmin(admin.ModelAdmin):
    list_display = ('id', 'name',
                    'identifier', 'status', 'deleted_flag')
    list_filter = ('status',)
    search_fields = ['name', 'identifier']


@admin.register(models.CompanyStatus)
class CompanyStatusAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'default_flag', 'inactive_flag')


@admin.register(models.CompanyType)
class CompanyTypeAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'vendor_flag', 'service_alert_flag')
    search_fields = ['name']


@admin.register(models.ContactType)
class ContactTypeAdmin(admin.ModelAdmin):
    list_display = ('id', 'description', 'default_flag', 'service_alert_flag')
    search_fields = ['description']


@admin.register(models.Contact)
class ContactAdmin(admin.ModelAdmin):
    list_display = ('id', 'full_name')
    search_fields = ('id', 'first_name', 'last_name')
    list_filter = ('company__name', )

    def full_name(self, obj):
        return str(obj)

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        return qs.select_related('company')


@admin.register(models.ContactCommunication)
class ContactCommunicationAdmin(admin.ModelAdmin):
    list_display = ('id', 'contact', 'value')
    search_fields = ['contact__first_name', 'contact__last_name', 'value']
    list_filter = ('contact__company__name', )

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        return qs.select_related('contact', 'contact__company')


@admin.register(models.CommunicationType)
class CommunicationTypeAdmin(admin.ModelAdmin):
    list_display = ('id', 'description', 'phone_flag', 'email_flag')
    search_fields = ['id', 'description']


@admin.register(models.ScheduleType)
class ScheduleTypeAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'identifier')
    search_fields = ['name']


@admin.register(models.ScheduleStatus)
class ScheduleStatusAdmin(admin.ModelAdmin):
    list_display = ('id', 'name')
    search_fields = ['name']


@admin.register(models.ScheduleEntry)
class ScheduleEntryAdmin(admin.ModelAdmin):
    list_display = ('id', 'ticket_object', 'activity_object', 'member',
                    'done_flag',
                    'status', 'date_start', 'date_end',)
    list_filter = ('status', 'done_flag', 'member')
    search_fields = ['name', 'ticket_object__id',
                     'activity_object__id', 'member__identifier']

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        return qs.select_related(
            'ticket_object', 'activity_object', 'member', 'status'
        )


@admin.register(models.TimeEntry)
class TimeEntryAdmin(admin.ModelAdmin):
    list_display = ('id', 'company', 'charge_to_id', 'charge_to_type',
                    'member', 'billable_option', 'actual_hours',
                    'time_start', 'time_end', 'hours_deduct', 'notes',
                    'internal_notes')
    list_filter = ('member', 'charge_to_type')
    search_fields = ['id', 'charge_to_id__id', 'member__identifier',
                     'charge_to_type', 'notes']

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        return qs.select_related('company', 'member', 'charge_to_id')


@admin.register(models.Team)
class TeamAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'board')
    search_fields = ['id', 'name']


@admin.register(models.TicketPriority)
class TicketPriorityAdmin(admin.ModelAdmin):
    model = models.TicketPriority
    list_display = ('id', 'name', 'sort', 'color')


@admin.register(models.ProjectStatus)
class ProjectStatusAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'default_flag', 'inactive_flag',
                    'closed_flag')
    list_filter = ('default_flag', 'inactive_flag', 'closed_flag',)
    search_fields = ['name']


@admin.register(models.ProjectType)
class ProjectTypeAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'default_flag', 'inactive_flag')
    list_filter = ('default_flag', 'inactive_flag',)
    search_fields = ['name']


@admin.register(models.ProjectPhase)
class ProjectPhaseAdmin(admin.ModelAdmin):
    list_display = ('id', 'description', 'project')
    search_fields = ['description', 'project__name']


@admin.register(models.ProjectRole)
class ProjectRoleAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'manager_role_flag', 'default_contact_flag')
    list_filter = ('manager_role_flag', 'default_contact_flag',)
    search_fields = ['name']


@admin.register(models.ProjectTeamMember)
class ProjectTeamMemberAdmin(admin.ModelAdmin):
    list_display = ('id', 'member', 'project', 'start_date', 'end_date')
    search_fields = [
        'member__id', 'member__first_name', 'member__last_name',
        'member__identifier', 'project__name', 'project__id'
    ]


@admin.register(models.Project)
class ProjectAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'status', 'type', 'description')
    list_filter = ('status', 'type')
    search_fields = ['name', 'description']


@admin.register(models.OpportunityStage)
class OpportunityStageAdmin(admin.ModelAdmin):
    list_display = ('id', 'name')


@admin.register(models.OpportunityStatus)
class OpportunityStatusAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'won_flag', 'lost_flag',
                    'closed_flag', 'inactive_flag')
    list_filter = ('won_flag', 'lost_flag', 'closed_flag', 'inactive_flag')
    search_fields = ['name']


@admin.register(models.OpportunityPriority)
class OpportunityPriorityAdmin(admin.ModelAdmin):
    list_display = ('id', 'name')


@admin.register(models.OpportunityType)
class OpportunityTypeAdmin(admin.ModelAdmin):
    list_display = ('id', 'description', 'inactive_flag')


@admin.register(models.Opportunity)
class OpportunityAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'stage', 'status', 'opportunity_type')
    list_filter = ('stage', 'status', 'opportunity_type')
    search_fields = ['name']


class ScheduleEntryInline(admin.StackedInline):
    model = models.ScheduleEntry


@admin.register(models.Ticket)
class TicketAdmin(admin.ModelAdmin):
    list_display = ('id', 'summary', 'status', 'record_type',)
    list_filter = ('record_type',)
    search_fields = \
        ['id', 'summary', 'company__name', 'status__name', 'board__name']
    autocomplete_fields = ['status', 'type', 'team']
    inlines = [
        ScheduleEntryInline
    ]

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        return qs.select_related('status', 'status__board')


@admin.register(models.ActivityStatus)
class ActivityStatusAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'inactive_flag', 'closed_flag')
    list_filter = ('inactive_flag', 'closed_flag')
    search_fields = ['name']


@admin.register(models.ActivityType)
class ActivityTypeAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'points', 'inactive_flag')
    list_filter = ('inactive_flag',)
    search_fields = ['name']


@admin.register(models.Activity)
class ActivityAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'notes', 'date_start', 'date_end',
                    'assign_to', 'opportunity', 'ticket')
    search_fields = [
        'name', 'notes', 'ticket__summary', 'opportunity__name',
        'company__name'
    ]
    list_filter = ('status', 'type')

    formfield_overrides = {
        db_models.CharField: {'widget': TextInput(attrs={'size': '40'})}
    }

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        return qs.select_related('ticket', 'opportunity', 'assign_to')


@admin.register(models.SalesProbability)
class SalesProbabilityAdmin(admin.ModelAdmin):
    list_display = ('id', 'probability',)


@admin.register(models.Calendar)
class CalendarAdmin(admin.ModelAdmin):
    list_display = ('id', 'name',)
    search_fields = ['name']


@admin.register(models.MyCompanyOther)
class MyCompanyOtherAdmin(admin.ModelAdmin):
    list_display = ('id', 'default_calendar',)


@admin.register(models.Holiday)
class Holiday(admin.ModelAdmin):
    list_display = ('id', 'name', 'date')
    search_fields = ['name', 'date']


@admin.register(models.HolidayList)
class HolidayList(admin.ModelAdmin):
    list_display = ('id', 'name',)
    search_fields = ['name']


@admin.register(models.Type)
class Type(admin.ModelAdmin):
    list_display = ('id', 'name', 'board',)
    search_fields = ['name']


@admin.register(models.SubType)
class SubType(admin.ModelAdmin):
    list_display = ('id', 'name', 'board',)
    search_fields = ['name']


@admin.register(models.Item)
class Item(admin.ModelAdmin):
    list_display = ('id', 'name', 'board',)
    search_fields = ['name']


@admin.register(models.TypeSubTypeItemAssociation)
class TypeSubTypeItemAssociationAdmin(admin.ModelAdmin):
    list_display = ('id', 'board', 'type', 'sub_type', 'item')
    search_fields = [
        'board__name', 'type__name', 'sub_type__name', 'item__name']


@admin.register(models.WorkType)
class WorkType(admin.ModelAdmin):
    list_display = ('id', 'name', 'inactive_flag',)
    search_fields = ['name']


@admin.register(models.WorkRole)
class WorkRole(admin.ModelAdmin):
    list_display = ('id', 'name', 'inactive_flag',)
    search_fields = ['name']


@admin.register(models.Agreement)
class Agreement(admin.ModelAdmin):
    list_display = ('id', '__str__', 'company', 'agreement_status',
                    'cancelled_flag', 'bill_time', 'work_type', 'work_role')
    list_filter = ['agreement_status', 'company']
    search_fields = ['name']


@admin.register(models.Source)
class Source(admin.ModelAdmin):
    list_display = ('id', 'name', 'default_flag',)
    search_fields = ['name']


@admin.register(models.TicketUDF)
class TicketUDFAdmin(admin.ModelAdmin):
    list_display = ('id', 'caption', 'type', 'entry_method',
                    'number_of_decimals')
    search_fields = ['caption', 'type']


@admin.register(models.ProjectUDF)
class ProjectUDFAdmin(admin.ModelAdmin):
    list_display = ('id', 'caption', 'type', 'entry_method',
                    'number_of_decimals')
    search_fields = ['caption', 'type']


@admin.register(models.ActivityUDF)
class ActivityUDFAdmin(admin.ModelAdmin):
    list_display = ('id', 'caption', 'type', 'entry_method',
                    'number_of_decimals')
    search_fields = ['caption', 'type']


@admin.register(models.OpportunityUDF)
class OpportunityUDFAdmin(admin.ModelAdmin):
    list_display = ('id', 'caption', 'type', 'entry_method',
                    'number_of_decimals')
    search_fields = ['caption', 'type']


@admin.register(models.CompanyNoteType)
class CompanyNoteTypeAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'identifier', 'default_flag')
    search_fields = ['name', 'identifier']


@admin.register(models.CompanyTeamRole)
class CompanyTeamRoleAdmin(admin.ModelAdmin):
    list_display = ('id', 'name')
    search_fields = ['name']


@admin.register(models.CompanyTeam)
class CompanyTeamAdmin(admin.ModelAdmin):
    list_display = ('id', 'company', 'team_role')


@admin.register(models.CompanySite)
class CompanySiteAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'company', 'inactive')


@admin.register(models.ConfigurationStatus)
class ConfigurationStatusAdmin(admin.ModelAdmin):
    list_display = ('id', 'description', 'closed_flag', 'default_flag')
    list_filter = ('closed_flag', 'default_flag')
    search_fields = ['description']


@admin.register(models.ConfigurationType)
class ConfigurationTypeAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'inactive_flag', 'system_flag')
    list_filter = ('inactive_flag', 'system_flag')
    search_fields = ['name']



# Generated by Django 2.1 on 2019-07-10 09:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0087_auto_20190705_1015'),
    ]

    operations = [
        migrations.AlterField(
            model_name='timeentry',
            name='billable_option',
            field=models.CharField(choices=[('Billable', 'Billable'), ('DoNotBill', 'Do Not Bill'), ('NoCharge', 'No Charge')], max_length=250),
        ),
    ]



# Generated by Django 2.1.14 on 2020-08-24 13:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djconnectwise', '0120_auto_20200721_1617'),
    ]

    operations = [
        migrations.AddField(
            model_name='syncjob',
            name='skipped',
            field=models.PositiveIntegerField(null=True),
        ),
    ]
